import React, {
  useEffect,
  useRef,
  useState,
} from 'react';
import {
  Text,
  View,
} from 'react-native';
import * as Notifications from 'expo-notifications';
import * as Device from 'expo-device';

Notifications.setNotificationHandler({
  handleNotification: async () => ({
    shouldShowAlert: true,
    shouldPlaySound: true,
    shouldSetBadge: true,
  }),
});

const NotificationHandler = ({ navigation }) => {
  const [expoPushToken, setExpoPushToken] = useState('');
  const [notification, setNotification] = useState(false);
  const [response, setResponse] = useState(false);
  const notificationListener = useRef();
  const responseListener = useRef();

  // Get last received notification
  const lastNotificationResponse = Notifications.useLastNotificationResponse();

  // Run on each new last received notification
  useEffect(() => {
    if (
      lastNotificationResponse &&
      lastNotificationResponse.notification.request.content.data.route &&
      lastNotificationResponse.actionIdentifier === Notifications.DEFAULT_ACTION_IDENTIFIER
    ) {
      // Handle notification data on notification press
      const data = lastNotificationResponse.notification.request.content.data;

      // Navigate to specified route
      navigation.navigate(data.route);
    }
  }, [lastNotificationResponse]);

  // Run on init
  useEffect(() => {
    // Get ExponentPushToken
    registerForPushNotificationsAsync()
      .then(token => {
        setExpoPushToken(token);
        // TODO: Send pushToken to AWS
      });

    // This listener is fired whenever a notification is received while the app is foregrounded
    notificationListener.current = Notifications.addNotificationReceivedListener(notification => {
      setNotification(notification);
    });

    // This listener is fired whenever a user taps on or interacts with a notification (works when app is foregrounded, backgrounded, or killed)
    responseListener.current = Notifications.addNotificationResponseReceivedListener(response => {
      setResponse(response);
    });

    return () => {
      Notifications.removeNotificationSubscription(notificationListener.current);
      Notifications.removeNotificationSubscription(responseListener.current);
    };
  }, []);

  // TODO: Return notification status message
  // return error if no expo token
  return (
    <View
      style={{
        position: 'absolute',
        bottom: 0,
        left: 0,
        padding: 10,
      }}
    >
      <Text
        style={{
          fontSize: 20,
        }}
      >
        {
          !expoPushToken
            ? <Text style={{color: 'red', fontSize: 14}}>Kan ikke motta notifikasjoner!</Text>
            : null
        }
      </Text>
    </View>
  );
}

async function registerForPushNotificationsAsync() {
  let token;
  if (Device.isDevice) {
    const { status: existingStatus } = await Notifications.getPermissionsAsync();
    let finalStatus = existingStatus;
    if (existingStatus !== 'granted') {
      const { status } = await Notifications.requestPermissionsAsync();
      finalStatus = status;
    }
    if (finalStatus !== 'granted') {
      alert('Failed to get push token for push notification!');
      return;
    }
    token = (await Notifications.getExpoPushTokenAsync()).data;
    console.log(token);
  } else {
    alert('Must use physical device for Push Notifications');
  }

  if (Platform.OS === 'android') {
    Notifications.setNotificationChannelAsync('default', {
      name: 'default',
      importance: Notifications.AndroidImportance.MAX,
      vibrationPattern: [0, 250, 250, 250],
      lightColor: '#FF231F7C',
    });
  }

  return token;
}

export default NotificationHandler;
