import { createSlice } from '@reduxjs/toolkit';

import Meal from 'src/global/mock-data/meal.mock.js';
import { newMealDay, newMealPlan } from 'src/global/mock-data/meal-plan/meal-plan.template.js';

const mealSlice = createSlice({
    name: 'meal',
    initialState: Meal,
    // initialState: {
    //     meal: 'Meal12',
    // },

    reducers: {
        /**
        * Creates a new plan/week.
        *
        * @param {Object} state - The current state object.
        * @param {Object} Meal - The Meal mock data object. Used as a template for new plans. 
        * @param {object} action - The action object containing the payload to update the meal plan.
        * @param {number} action.payload.id - The new ID of the meal plan to create.
        * @returns {void}
        */
        createPlan: (state, action) => {
            const indexOfPlan = state.plans.findIndex(plan => plan.id === action.payload.planId)
            if (indexOfPlan !== -1) {
                console.log('Plan already exists');
                return;
            }
            const newWeek = newMealPlan(action.payload.planId);

            const lastplan = state.plans[0];
            const lastDay = state.plans[state.plans.length - 1].days[6];
            
            state.plans.push(newWeek);
        },

        /**
        * Updates a meal plan in the state or adds it if it doesn't exist.
        *
        * @param {object} state - The current state object.
        * @param {object} action - The action object containing the payload to update the meal plan.
        * @param {number} action.payload.id - The ID of the meal plan to update.
        * @param {object} action.payload - The updated meal plan object.
        * @returns {void}
        */
        setPlan: (state, action) => {
            // indexOfPlan = state.plans.findIndex(plan => plan.id === action.payload.plan.id )
            // if (indexOfPlan !== -1) {
            //     state.plans[indexOfPlan] = action.payload.plan
            // }
            // else state.plans.push(action.payload.plan)
        },

        /**
        * Delete a meal plan from the state.
        *
        * @param {object} state - The current state of the application.
        * @param {object} action - The action object containing the plan ID to delete.
        * @param {string} action.payload.id - The ID of the plan to delete.
        * @returns {void}
        */
        deletePlan: (state, action) => {
            indexOfPlan = state.plans.findIndex(plan => plan.id === action.payload.plan.id)
            if (indexOfPlan !== -1) {
                state.plans.splice(indexOfPlan, 1)
            }
        },
        
        /**
         * Creates a new day in the specified meal plan.
         * 
         * @param {Object} state - The current state object.
         * @param {Object} action - The action object containing the payload to create a new day.
         * @param {string} action.payload.planId - The ID of the meal plan to update.
         * @returns {void}
         */ 
        createDay: (state, action) => {
            const newDay = newMealDay;
            const indexOfPlan = state.plans.findIndex(plan => plan.id === action.payload.planId)
            if (indexOfPlan === -1) {
                console.log('Plan not found');
                return;
            }
            if (state.plans[indexOfPlan].days.length < 7) {
                state.plans[indexOfPlan].days.push(newDay)
            } else {
                // week is full
                // should never trigger here
            }
        },

        /**
         * Creates a new meal in the specified meal plan.
         * If dayIndex and mealIndex are 0, a new plan (week/plan[i]) is created.
         * If mealIndex is 0, a new day (days[i]) is created.
         * 
         * @param {Object} state - The current state object.
         * @param {Object} action - The action object containing the payload to create a new day.
         * @param {string} action.payload.planId - The ID of the meal plan to update.
         * @param {number} action.payload.dayIndex - The index of the day within the plan.
         * @param {number} action.payload.mealIndex - The index of the meal within the day.
         * @param {Object} action.payload.meal - The updated meal object.
         * @returns {void}
         */
        createMeal: (state, action) => {
            // const indexOfPlan = state.plans.findIndex(plan => plan.id === action.payload.planId)
            // if (indexOfPlan === -1) {
            //     return;
            // }
            // if (action.payload.dayIndex === 0 && mealIndex === 0) { // new week
            //     const newWeek = {
            //         id: state.plans[indexOfPlan].days.length + 1,
            //         createdAt: new Date().toLocaleString(), // toISOString()
                    

            //     }
            //     state.plans.push();
            //     // state.plans[indexOfPlan].days.push(action.payload.day)
            // }
            // state.plans[indexOfPlan].days[action.payload.dayIndex].meals.push(action.payload.meal)
        },

        /**
        * Updates a specific meal in the state.
        * 
        * @param {Object} state - The current state object.
        * @param {Object} action - The action object containing the meal to update.
        * @param {string} action.payload.planId - The ID of the meal plan to update.
        * @param {number} action.payload.dayIndex - The index of the day within the plan.
        * @param {number} action.payload.mealIndex - The index of the meal within the day.
        * @param {Object} action.payload.meal - The updated meal object.
        * @returns {void}
        */
        setMeal: (state, action) => { 
            const indexOfPlan = state.plans.findIndex(plan => plan.id === action.payload.planId)
            if (indexOfPlan !== -1) {
                state.plans[indexOfPlan].days[action.payload.dayIndex].meals[action.payload.mealIndex] = action.payload.meal
            }
        },

        /**
        * Delete a meal from the specified day in a meal plan.
        * 
        * @param {Object} state - The current state of the application.
        * @param {Object} action - The action containing the meal plan ID, day index, and meal index.
        * @returns {undefined}
        */
        deleteMeal : (state, action) => {
            indexOfPlan = state.plans.findIndex(plan => plan.id === action.payload.planId)
            if (indexOfPlan !== -1) {
                state.plans[indexOfPlan].days[action.payload.dayIndex].meals.splice(action.payload.mealIndex, 1)
            }
        },
    },
})

export const {
    createPlan,
    setPlan,
    deletePlan,
    createDay,
    setMeal,
    deleteMeal,
} = mealSlice.actions;

export default mealSlice.reducer;