import React, {
    useEffect,
    useState,
} from 'react';
import {
    Pressable,
    StyleSheet,
    Text,
    TouchableOpacity,
    View,
} from 'react-native';
import {
    useDispatch,
    useSelector,
} from 'react-redux';
import { Entypo } from '@expo/vector-icons';
import { Audio } from 'expo-av';

import { unpackObject } from 'src/global/reducers/packing/OBJECTSLICE';
import { removeFromBackpack } from 'src/global/reducers/packing/packedObjects.slice';
import { selectObject, deselectObject } from 'src/global/reducers/packing/selectedObject.slice';

import BackpackList from 'src/global/mock-data/objects.list';
import SelectedObject from './backpack/backpack.selectedObject';

import { EmptyPackedList, EmptyUnpackedList } from './backpack/backpack.empty-state';
import BackpackInfo from './backpack/backpack.info';
import BackButton from 'src/global/components/buttons/BackButton';
import * as colorPalette from 'src/global/constants/colorPalette';
import fonts from 'src/assets/fonts/fonts';
import metrics from 'src/global/constants/metrics';

const BackpackView = ({ navigation }) => {
    const { objects } = useSelector((state) => state.allObjects);
    const { packedObjects } = useSelector((state) => state.packedObjects);
    const { selectedObject } = useSelector((state) => state.selectedObject);
    const dispatch = useDispatch();

    const [unpackedObjects, setUnpackedObjects] = useState([]);
    // Current object list used for object range
    const [currentObjectList, setCurrentObjectList] = useState(packedObjects);

    useEffect(() => {
        updateUnpackedList();
    }, []);

    // Loop over all objects and push to array if required and not packed
    // Write new array to unpackedObjects
    const updateUnpackedList = () => {
        let objectList = [];

        objects
            .map(object => {
                if (object.features.required && !object.features.packed) {
                    objectList.push(object);
                }
            });

        setUnpackedObjects(objectList);
    }

    // true: Viewing packed objects
    // false: Viewing unpacked objects
    const [packedView, setPackedView] = useState(true);

    // Save a selected object in redux store
    const saveSelectedObject = (object) => {
        dispatch(selectObject(object));
    }

    // Remove an object from packedObjects array
    // Toggle packed on object in allObjects array
    // Add removed object to displayed list
    const removeItemFromBackpack = (object) => {
        dispatch(unpackObject(object.id));
        dispatch(removeFromBackpack(object));
        dispatch(deselectObject());
        
        const removedObject = objects.find(o => o.id === object.id);

        // Bug: return [removedObject, ...list];
        // Object is placed at top of array but does not render with grayed out color
        if (removedObject) {
            setUnpackedObjects((list) => {
                return [...list, removedObject];
            });
        }
    }

    

    // Unload, load and play the selected object sound
    // Pass the function down to <SelectedObject /> component
    const [currentSound, setCurrentSound] = useState()

    const SoundPlayer = (name, sound) => {
        if (currentSound == null) {
            console.log('Sound is undefined');
        } else {
            currentSound.sound.unloadAsync().catch(()=>{});
        }

        console.log('Playing: ' + name);
        Audio.Sound.createAsync(
            sound,
            {shouldPlay: true}
        ).then((res)=>{
            setCurrentSound(res);
            res.sound.setOnPlaybackStatusUpdate((status)=>{
    
                if(!status.didJustFinish) return;
                console.log('Unloading ' + name);
    
                res.sound.unloadAsync().catch(()=>{});
            });
        }).catch((error)=>{});
    }

    // const [selectedObjectSound, setSelectedObjectSound] = useState(new Audio.Sound());
    // const playSelectedObjectSound = async (sound) => {
        // await selectedObjectSound.unloadAsync();
        // await selectedObjectSound.loadAsync(sound);
        // await selectedObjectSound.playAsync();
    // }

    // The range of objects displayed in list
    // Range used for both packed and unpacked list
    const [objectRange, setObjectRange] = useState({
      start: 0,
      end: 6,
    });
  
    // Change range, set type 'next' or 'prev'
    const setRange = (type) => {
      switch (type) {
          case 'next':
              rangeNext();
              break;
          case 'prev':
              rangePrev();
              break;
      }
    }

    // Set ranges +/- 6
    const rangeNext = () => {
        if (objectRange.end < currentObjectList.length) {
            setObjectRange({
                start: objectRange.start + 6,
                end: objectRange.end + 6
            });
        }
    }
    const rangePrev = () => {
        if (objectRange.start > 0) {
            setObjectRange({
                start: objectRange.start - 6,
                end: objectRange.end - 6
            });
        }
    }

    return (
        <View style={styles.container}>
            
            <View style={styles.innerContainer}>

                {/* Action row in orange area */}
                <View style={styles.actionRow}>

       
                </View>

                {
                    // Previous range button
                    objectRange.start > 0
                        // If range start is larger than 0, show pagination arrow
                        ? <TouchableOpacity onPress={() => setRange('prev')}
                                style={[styles.rangeButton, {
                                    position: 'absolute',
                                    top: '4%',
                                    left: '29%'
                                }]}
                        >
                            <Entypo name='chevron-small-up' size={fonts.size.font18} color='black' />
                        </TouchableOpacity>
                        // Hide pagination arrow if range start is 0 or less
                        : null
                }

                <View style={styles.contentWrapper}>
                    {/* List tabs */}
                    <View style={styles.tabsContainer}>
                        {/* Display packed list */}
                        <Pressable onPress={() => {
                            setObjectRange({
                                start: 0,
                                end: 6
                            });
                            setCurrentObjectList(packedObjects);
                            setPackedView(true);
                        }}
                        style={[styles.tab, {
                            backgroundColor: packedView ? 'hsla(26, 69%, 86%, 1)' : 'hsla(26, 69%, 86%, .5)',
                            marginBottom: 4
                        }]}>
                            <Text style={styles.tabText}>Pakket</Text>
                        </Pressable>
                        {/* Display unpacked list */}
                        <Pressable onPress={() => {
                            setObjectRange({
                                start: 0,
                                end: 6
                            });
                            setCurrentObjectList(unpackedObjects);
                            setPackedView(false);
                        }}
                        style={[styles.tab, {
                            backgroundColor: packedView ? 'hsla(26, 69%, 86%, .5)' : 'hsla(26, 69%, 86%, 1)',
                        }]}>
                            <Text style={styles.tabText}>Ikke pakket</Text>
                        </Pressable>
                    </View>
                    <View style={styles.contentContainer}>

                        <View style={[styles.column, styles.objectListColumn]}>

                            {
                                packedView
                                    ? packedObjects.length > 0
                                        // If packed list not empty
                                        ? <BackpackList objects={packedObjects}
                                                        range={objectRange}
                                                        selectObject={saveSelectedObject}
                                                        selectedObjectID={selectedObject ? selectedObject.id : 'NA' }
                                                        requiredList={false}
                                        />
                                        // If packed list empty state
                                        : <EmptyPackedList />
                                    : unpackedObjects.length > 0
                                        // If required list not empty
                                        ? <BackpackList objects={unpackedObjects}
                                                        range={objectRange}
                                                        selectObject={saveSelectedObject}
                                                        selectedObjectID={selectedObject ? selectedObject.id : 'NA'}
                                                        requiredList={true}
                                        />
                                        // If required list empty state
                                        : <EmptyUnpackedList />
                            }

                        </View>

                        <View style={styles.column}>

                            {
                                selectedObject
                                    // If an object is selected 
                                    ? <SelectedObject object={selectedObject}
                                                      playSound={SoundPlayer}
                                                      removeObject={removeItemFromBackpack}
                                    />
                                    // If no object is selected state
                                    : <BackpackInfo />
                            }

                        </View>

                    </View>
                </View>

                {
                    // Next range button
                    objectRange.end < currentObjectList.length
                        // If the range end is less than the list length, show pagination arrow
                        ? <TouchableOpacity onPress={() => setRange('next')}
                                style={[styles.rangeButton, {
                                    position: 'absolute',
                                    bottom: '4%',
                                    left: '29%'
                                }]}
                        >
                            <Entypo name='chevron-small-down' size={fonts.size.font18} color='black' />
                        </TouchableOpacity>
                        // Hide pagination arrow if range end is larger than list length
                        : null
                }

            </View>
            <BackButton buttonColor={colorPalette.CORNSILK} onPress={() => navigation.goBack()}/>
        </View>
    );
}

const styles = StyleSheet.create({
    container: {
        flex: 1,
        padding: 24,
        paddingTop: '3%',
        backgroundColor: '#7c563f',
    },
    innerContainer: {
        flex: 1,
        alignItems: 'center',
        paddingTop: '1%',
        backgroundColor: '#e3a06b',
        borderRadius: metrics.screenHeight / 20,
    },
    actionRow: {
        width: '90%',
        height: '4%',
        // marginBottom: '1%',
    },
    contentWrapper: {
        width: '90%',
        height: '90%',
        flexDirection: 'row'
    },
    tabsContainer: {
        // width: 42,
        width: metrics.screenRatio *80,
        height: '100%',
        justifyContent: 'center',
    },
    tab: {
        height: '35%',
        borderTopLeftRadius: 20,
        borderBottomLeftRadius: 20,
        alignItems: 'center',
        justifyContent: 'center',
        backgroundColor: '#ECBD98',
        // borderRightWidth: 2,
        // borderColor: '#F0CBAE',
    },
    tabText: {
        width: '300%',
        fontSize: fonts.size.font12,
        textAlign: 'center',
        transform: [
            { rotateZ: '-90deg' }
        ],
        fontWeight: 'bold'
    },
    contentContainer: {
        flex: 1,
        padding: 10,
        flexDirection: 'row',
        backgroundColor: 'hsla(26, 69%, 86%, 1)',
        borderRadius: metrics.screenHeight / 20,
        // elevation: 1,
    },
    column: {
        flex: 1,
    },
    objectListColumn: {
        borderRightWidth: 1,
        borderColor: '#7c563f'
    },
    rangeButton: {
        zIndex: 9999,
        width: metrics.screenHeight / 25,
        height: metrics.screenHeight /25,
        alignItems: 'center',
        justifyContent: 'center',
        alignSelf: 'center',
        backgroundColor: '#fff',
        borderRadius: 100,
        elevation: 5,
    },
});

export default BackpackView;
